CLevel = {};

function CLevel:new()
--
	local object = 
	{
		loader,
		map,
		slopeHeights = {},
		jumpDir = CPoint:new(),
		jumpAngle = 0,		
		slopeJumpMod
	};
	
	object.loader = require("ATL.Loader");
	object.loader.path = "maps/" --Change this to wherever your .tmx files are
	object.map = object.loader.load("tilemap.tmx") --Change this to the name of your mapfile
	object.map.useSpriteBatch = true;
	
	setmetatable( object, { __index = CLevel} );
	
	return object;	
--
end;

function CLevel:GetDownwardSlopedCollisions( extLookdown, x1, x2, y1, y2 )
--
	-- now we need the tile positions on the map
	local cy1 = floor( y1 / self.map.tileHeight );
	
	-- ext look down is how far to look down so the sprite can "stick" to the map
	local cy2 = floor( y2 / self.map.tileHeight ) + extLookdown;
		
	local cx1 = floor( x1 / self.map.tileWidth );
	local cx2 = floor( x2 / self.map.tileWidth );
	
	local floorPos = nil;
	
	for loopCy = cy1, cy2 do
	--
		floorPos = nil;
		for loopCx = cx1, cx2 do
		--
			tile = self.map( "CollLayer" )( loopCx, loopCy );
			if( tile )then
			--
				local tileType = tile.properties.type;
				if( tileType == "slope" )then

					local xOverlap = nil;
					
					-- checking for flipped tiles tells which side we should bias the x overlap on
					local isFlipped = IsMapLayerTileFlippedHorz( self.map( "CollLayer" ), loopCx, loopCy );
					if( not( isFlipped ) )then
						-- how far into the current tile we are.  Since we subtracted 1 from 
						-- the width, we need to add it back in for the proper overlap
						xOverlap = floor( x2 - ( loopCx * self.map.tileWidth ) ) + 1;
						if( xOverlap > 16 )then	xOverlap = 16; end;
						
						-- only want to set this if the right most tile has collided
						if( loopCx == cx2 )then
							if( tile.properties.un_n_x and tile.properties.un_n_y )then
								self.jumpDir:Set( tile.properties.un_n_x, tile.properties.un_n_y );
							end;
							if( tile.properties.jump_angle_n )then
								self.jumpAngle = tile.properties.jump_angle_n;
							end;							
						end;
					else
						xOverlap = floor( ( ( loopCx * self.map.tileWidth ) + self.map.tileWidth ) - x1 ) + 1;
						if( xOverlap > 16 )then	xOverlap = 16; end;
						-- only want to set this if the left most tile has collided
						if( loopCx == cx1 )then
							if( tile.properties.un_f_x and tile.properties.un_f_y )then
								self.jumpDir:Set( tile.properties.un_f_x, tile.properties.un_f_y );
							end;
							if( tile.properties.jump_angle_f )then
								self.jumpAngle = tile.properties.jump_angle_f;
							end;														
						end;
					end;
						
					local floorHeight = nil;

					-- next up we want the heightmap of the tile
					if( self.slopeHeights[ tile.id ] )then
						floorHeight = self.slopeHeights[ tile.id ][ xOverlap ];
					end;
					
					if( floorHeight )then
						local tileBase = ( ( loopCy * self.map.tileHeight ) + self.map.tileHeight );
						local floorTop = tileBase - floorHeight;

                                                -- extLookdown is how many tiles, so we multiply
                                                -- by tile height to get the amount of pixels
                                                local lookDown = ( extLookdown * self.map.tileHeight );
                                                -- extendedLookdown must be at least 1 to get it to 
						-- stick to the slope and yet it snaps to the slope 
						-- before the sprite actually gets there.
                                                if( ( y2 + lookDown ) > floorTop )then
                                                        if( not( floorPos ) )then
                                                                floorPos = floorTop;
                                                        elseif( floorPos > floorTop )then
                                                                floorPos = floorTop;
                                                        end;
                                                end;						
					end;
				end; -- if tileType
			--
			end; -- if tile
		--
		end; -- for loopCx
		
		if( floorPos )then
			break;
		end;
	--
	end; -- for loopCy
	
	return( floorPos );
--
end;

function CLevel:LeftsideSlopeCheck( x1, x2, y )
--
	-- now we need the tile positions on the map
	local cx1 = floor( x1 / self.map.tileWidth );
	local cx2 = floor( x2 / self.map.tileWidth );
	
	local widthInTiles = floor( ( cx1 - cx2 ) / 2 );
	if( widthInTiles < 1 )then
		widthInTiles = 1;
	end;
	
	-- even though we're considering rightward movement
	-- we have to cover all the tiles the sprite takes up
	-- plus the tiles the movement takes up.  So we also
	-- need the vertical tiles.
	--local y = ( self:GetYPos() + self:GetHeight() ) - 1;
	
	local cy2 = floor( y / self.map.tileHeight );
	local cy1 = cy2 - widthInTiles;
	
	local floorPos = nil;
	
	self.slopeJumpMod = 0;
	
	for loopCx = cx1, cx2, -1 do
	--
		for loopCy = cy1, cy2 do
		--
			tile = self.map( "CollLayer" )( loopCx, loopCy );
			if( tile )then
			--
				local tileType = tile.properties.type;
				if( tileType == "slope" )then
				--
					local xOverlap = nil;
					
					-- checking for flipped tiles tells which side we should bias the x overlap on
					local isFlipped = IsMapLayerTileFlippedHorz( self.map( "CollLayer" ), loopCx, loopCy );
					if( isFlipped )then
						-- how far into the current tile we are. 
						xOverlap = ( floor( ( ( loopCx * self.map.tileWidth ) + self.map.tileWidth ) - x2 ) ) + 1;
						if( xOverlap > 16 )then	xOverlap = 16; end;						
					end;
					
					local floorHeight = nil;

					-- next up we want the heightmap of the tile
					if( self.slopeHeights[ tile.id ] )then
						floorHeight = self.slopeHeights[ tile.id ][ xOverlap ];
					end;

					if( floorHeight )then
						local tileBase = ( ( loopCy * self.map.tileHeight ) + self.map.tileHeight );
						local floorTop = tileBase - floorHeight;
						if( not( floorPos ) )then
							floorPos = floorTop;
						elseif( floorPos > floorTop )then
							floorPos = floorTop;
						end;

						local jumpMod = tile.properties.jump_mod;
						if( jumpMod )then
							self.slopeJumpMod = jumpMod;
						end;
					end;
				--	
				end; -- if tileType
			--
			end; -- if tile
			
		--
		end; -- for loopCy
		
	--
	end; -- for loopCx

	return( floorPos );
--
end;

function CLevel:RightsideSlopeCheck( x1, x2, y )
--
	-- now we need the tile positions on the map
	local cx1 = floor( x1 / self.map.tileWidth );
	local cx2 = floor( x2 / self.map.tileWidth );

	local widthInTiles = floor( ( cx2 - cx1 ) / 2 );
	if( widthInTiles < 1 )then
		widthInTiles = 1;
	end;
	
	local cy2 = floor( y / self.map.tileHeight );
	local cy1 = cy2 - widthInTiles;

	self.slopeJumpMod = 0;

	local floorPos = nil;

	for loopCx = cx1, cx2 do
	--
		for loopCy = cy1, cy2 do
		--
			tile = self.map( "CollLayer" )( loopCx, loopCy );
			if( tile )then
			--
				local tileType = tile.properties.type;
				if( tileType == "slope" )then
				--
					local xOverlap = nil;
					
					-- checking for flipped tiles tells which side we should bias the x overlap on
					local isFlipped = IsMapLayerTileFlippedHorz( self.map( "CollLayer" ), loopCx, loopCy );
					if( not( isFlipped ) )then
						-- how far into the current tile we are.  Since we subtracted 1 from 
						-- the width, we need to add it back in for the proper overlap
						xOverlap = floor( x2 - ( loopCx * self.map.tileWidth ) );
						if( xOverlap > 16 )then	xOverlap = 16; end;
					end;
					
					local floorHeight = nil;

					-- next up we want the heightmap of the tile
					if( self.slopeHeights[ tile.id ] )then
						floorHeight = self.slopeHeights[ tile.id ][ xOverlap ];
					end;

					if( floorHeight )then
						local tileBase = ( ( loopCy * self.map.tileHeight ) + self.map.tileHeight );
						local floorTop = tileBase - floorHeight;
						if( not( floorPos ) )then
							floorPos = floorTop;
						elseif( floorPos > floorTop )then
							floorPos = floorTop;
						end;
						
						local jumpMod = tile.properties.jump_mod;
						if( jumpMod )then
							self.slopeJumpMod = jumpMod;
						end;
					end;
				--	
				end; -- if tileType
			--
			end; -- if tile			
		--
		end; -- for loopCy
	--
	end; -- for loopCx
	
	return( floorPos );
--
end;

function CLevel:GetLeftsideCollision( x1, x2, y1, y2 )
--
	-- now we need the tile positions on the map
	local cx1 = floor( x1 / self.map.tileWidth );
	local cx2 = floor( x2 / self.map.tileWidth );
	
	local cy1 = floor( y1 / self.map.tileHeight );
	local cy2 = floor( y2 / self.map.tileHeight );
	
	local wallPos = nil;
	for loopCx = cx1, cx2, -1 do
	--
		wallPos = nil;
		for loopCy = cy1, cy2 do
		--
			tile = self.map( "CollLayer" )( loopCx, loopCy );
			if( tile )then
			--
				local isBlocked = tile.properties.block_e;
				local tileType = tile.properties.type;
				if( isBlocked or tileType == "border" )then
				--
					-- this bit just makes sure we get the rightest tile position
					local tileRight = ( loopCx * self.map.tileWidth ) + self.map.tileWidth;
					if( not( wallPos ) )then
						wallPos = tileRight;
					elseif( wallPos < tileRight )then
						wallPos = tileRight;
					end;
				--
				end; -- if tileType
			--
			end; -- if tile
		--
		end; -- for loopCy
		
		if( wallPos )then
			break;
		end;
	--
	end; -- for loopCx
	
	return( wallPos );
--
end;

function CLevel:GetRightsideCollision( x1, x2, y1, y2 )
--
	-- now we need the tile positions on the map
	local cx1 = floor( x1 / self.map.tileWidth );
	local cx2 = floor( x2 / self.map.tileWidth );

	local cy1 = floor( y1 / self.map.tileHeight );
	local cy2 = floor( y2 / self.map.tileHeight );
	
	local wallPos = nil;

	for loopCx = cx1, cx2 do
	--
		wallPos = nil;
		for loopCy = cy1, cy2 do
		--
			tile = self.map( "CollLayer" )( loopCx, loopCy );
			if( tile )then
			--
				local isBlocked = tile.properties.block_w;
				local tileType = tile.properties.type;
				if( isBlocked or tileType == "border" )then
				--
					-- this bit just makes sure we get the leftest tile position
					local tileLeft = loopCx * self.map.tileWidth;
					if( not( wallPos ) )then
						wallPos = tileLeft;
					elseif( wallPos > tileLeft )then
						wallPos = tileLeft;
					end;
				--
				end; -- if tileType
			--
			end; -- if tile			
		--
		end; -- for loopCy
		
		if( wallPos )then
			break;
		end;
	--
	end; -- for loopCx

	return( wallPos );
--
end;

function CLevel:GetDownwardNonSlopedCollisions( x1, x2, y1, y2 )
--
	-- now we need the tile positions on the map
	local cy1 = floor( y1 / self.map.tileHeight );
	local cy2 = floor( y2 / self.map.tileHeight );
	
	local cx1 = floor( x1 / self.map.tileWidth );
	local cx2 = floor( x2 / self.map.tileWidth );
	
	local floorPos = nil;
	
	for loopCy = cy1, cy2 do
	--
		floorPos = nil;
		for loopCx = cx1, cx2 do
		--
			tile = self.map( "CollLayer" )( loopCx, loopCy );
			if( tile )then
			--
				local isBlocked = tile.properties.block_n;
				local tileType = tile.properties.type;
				if( isBlocked or tileType == "border" )then
				--
					-- this bit just makes sure we get the highest tile position
					local tileTop = loopCy * self.map.tileHeight;
					if( not( floorPos ) )then
						floorPos = tileTop;
					elseif( floorPos > tileTop )then
						floorPos = tileTop;
					end;
					self.jumpDir:Set( 0, -1 );
					self.jumpAngle = 90;
					self.slopeJumpMod = 0;
				--
				end; -- if tileType
			--
			end; -- if tile
		--
		end; -- for loopCx
		
		if( floorPos )then
			break;
		end;
	--
	end; -- for loopCy
	
	return( floorPos );
--
end;

function CLevel:GetUpwardCollisions( x1, x2, y1, y2 )
--
	-- now we need the tile positions on the map
	local cy1 = floor( y1 / self.map.tileHeight );
	local cy2 = floor( y2 / self.map.tileHeight );
	
	local cx1 = floor( x1 / self.map.tileWidth );
	local cx2 = floor( x2 / self.map.tileWidth );
	
	local ceilingPos = nil;
	
	for loopCy = cy1, cy2, -1 do
	--
		ceilingPos = nil;
		for loopCx = cx1, cx2 do
		--
			tile = self.map( "CollLayer" )( loopCx, loopCy );
			if( tile )then
			--
				local isBlocked = tile.properties.block_s;
				local tileType = tile.properties.type;
				if( isBlocked or tileType == "border" )then
				--
					-- this bit just makes sure we get the lowest tile position
					local tileBottom = ( loopCy * self.map.tileHeight ) + self.map.tileHeight;
					if( not( ceilingPos ) )then
						ceilingPos = tileBottom;
					elseif( ceilingPos < tileBottom )then
						ceilingPos = tileBottom;
					end;
				--
				end; -- if tileType
			--
			end; -- if tile
		--
		end; -- for loopCx
		
		if( ceilingPos )then
			break;
		end;
	--
	end; -- for loopCy
	
	return( ceilingPos );
--
end;

