CPlayer = {};

function CPlayer:new()
--
	local object = 
	{
		rect = CRect:new(),
		bounds = CRect:new(),
		speed = CPoint:new(),
		velocity = CPoint:new(),
		minVelocity = CPoint:new(),
		maxVelocity = CPoint:new(),
		acc = 0,
		dec = 0,
		jumpDir = CPoint:new(),
		jumpAngle = 0,
		jumpForce = 0,
		jumpVelocity = CPoint:new(),
		isJumping = "false",
		facing = "right",
		groundType = "none",
		state = "falling",
		
		-- slopeJumpMod is for a special case
		-- when one is moving uphill at high speeds
		-- in order to clear the slope by jumping
		slopeJumpMod = 0
		
	};
	
	setmetatable( object, { __index = CPlayer } );
	
	return object;
--
end;

function CPlayer:SetPos(x, y)
--
	self.rect:SetPos(x, y);
	self:KeepInBounds();
--
end;

function CPlayer:SetXPos( x )
--
	self.rect:SetXPos( x );
	self:KeepInHorzBounds();
--
end;

function CPlayer:SetYPos( y )
--
	self.rect:SetYPos( y );
	self:KeepInVertBounds();
--
end;

function CPlayer:GetPos()
--
	return self.rect:GetPos();		
--
end;

function CPlayer:GetXPos()
--
	return self.rect:GetXPos();
--
end;

function CPlayer:GetYPos()
--
	return self.rect:GetYPos();
--
end;

function CPlayer:SetSize(w, h)
--
	self.rect:SetSize( w, h );	
--
end;

function CPlayer:SetWidth( w )
--
	self.rect:SetWidth( w );	
--
end;

function CPlayer:SetHeight( h )
--
	self.rect:SetHeight( h );
--
end;

function CPlayer:GetSize()
--
	return self.rect:GetSize();		
--
end;

function CPlayer:GetWidth()
--
	return self.rect:GetWidth();
--
end;

function CPlayer:GetHeight()
--
	return self.rect:GetHeight();
--
end;

function CPlayer:SetSpeed(x, y)
--
	self.speed:Set(x, y);
--
end;

function CPlayer:SetXSpeed( x )
--
	self.speed:SetX( x );
--
end;

function CPlayer:SetYSpeed( y )
--
	self.speed:SetY( y );
--
end;

function CPlayer:GetSpeed()
--
	return( self.speed:Get() );		
--
end;

function CPlayer:GetXSpeed()
--
	return( self.speed:GetX() );
--
end;

function CPlayer:GetYSpeed()
--
	return( self.speed:GetY() );
--
end;

function CPlayer:SetAcceleration( acc )
--
	self.acc = acc;
--
end;

function CPlayer:SetDeceleration( dec )
--
	self.dec = dec;
--
end;

function CPlayer:GetAcceleration()
--
	return( self.acc );
--
end;

function CPlayer:GetDeceleration()
--
	return( self.dec );
--
end;

function CPlayer:SetBounds( x, y, w, h )
--
	self.bounds:SetPos( x, y );
	self.bounds:SetSize( w, h );
--
end;

function CPlayer:KeepInHorzBounds()
--
	local width = self:GetWidth();
	local left = self:GetXPos();
	local right = left + width;

	local left_bound = self.bounds:GetXPos();
	local right_bound = left_bound + self.bounds:GetWidth();
	
	if( left < left_bound )then
		self.rect:SetXPos( left_bound );
	end;
		
	if( right > right_bound )then
		self.rect:SetXPos( right_bound - width );
	end;
--
end;

function CPlayer:KeepInVertBounds()
--
	local height = self:GetHeight();
	local top = self:GetYPos();
	local bottom = top + height;
	local top_bound = self.bounds:GetYPos();
	local bottom_bound = top_bound + self.bounds:GetHeight();
	
	if( top < top_bound )then
		self.rect:SetYPos( top_bound );
	end;
	
	if( bottom > bottom_bound )then
		self.rect:SetYPos( bottom_bound - height );
	end;
--
end;

function CPlayer:KeepInBounds()
--
	self:KeepInHorzBounds();
	self:KeepInVertBounds();
--
end;

function CPlayer:SetJumpForce( jumpForce )
--
	self.jumpForce = jumpForce;
--
end;

function CPlayer:SetMinVelocity( xVel, yVel )
--
	self.minVelocity:Set( xVel, yVel );
--
end;

function CPlayer:SetMaxVelocity( xVel, yVel )
--
	self.maxVelocity:Set( xVel, yVel );
--
end;

function CPlayer:SetVel(x, y)
--
	local xVel = x;
	if( xVel < self.minVelocity:GetX() )then
		xVel = self.minVelocity:GetX();
	elseif( xVel > self.maxVelocity:GetX() )then
		xVel = self.maxVelocity:GetX();
	end;

	local yVel = y;
	if( yVel < self.minVelocity:GetY() )then
		yVel = self.minVelocity:GetY();
	elseif( yVel > self.maxVelocity:GetY() )then
		yVel = self.maxVelocity:GetY();
	end;
	
	self.velocity:Set( xVel, yvel );	
--
end;

function CPlayer:SetXVel( x )
--
	local xVel = x;
	if( xVel < self.minVelocity:GetX() )then
		xVel = self.minVelocity:GetX();
	elseif( xVel > self.maxVelocity:GetX() )then
		xVel = self.maxVelocity:GetX();
	end;

	self.velocity:SetX( xVel );
--
end;

function CPlayer:SetYVel( y )
--
	local yVel = y;
	if( yVel < self.minVelocity:GetY() )then
		yVel = self.minVelocity:GetY();
	elseif( yVel > self.maxVelocity:GetY() )then
		yVel = self.maxVelocity:GetY();
	end;

	self.velocity:SetY( yVel );
--
end;

function CPlayer:GetVel()
--
	return self.velocity:Get();		
--
end;

function CPlayer:GetXVel()
--
	return self.velocity:GetX();
--
end;

function CPlayer:GetYVel()
--
	return self.velocity:GetY();
--
end;

function CPlayer:SetJumpVel(x, y)
--	
	self.jumpVelocity:Set( x, y );	
--
end;

function CPlayer:SetXJumpVel( x )
--
	self.jumpVelocity:SetX( x );
--
end;

function CPlayer:SetYJumpVel( y )
--
	self.jumpVelocity:SetY( y );
--
end;

function CPlayer:GetJumpVel()
--
	return self.jumpVelocity:Get();		
--
end;

function CPlayer:GetXJumpVel()
--
	return self.jumpVelocity:GetX();
--
end;

function CPlayer:GetYJumpVel()
--
	return self.jumpVelocity:GetY();
--
end;

function CPlayer:GenerateMovementCommands( isLeftArrowDown, isRightArrowDown )
--
	local acc = self:GetAcceleration();
	local dec = self:GetDeceleration();
	if ( isLeftArrowDown ) then
	--	
		self.facing = "left";
		if( self:GetXVel() > 0 ) then
			acc = acc * 4;
		end;
		
		self:SetXVel( self:GetXVel() - acc );
	--
	elseif ( isRightArrowDown ) then
	--
		self.facing = "right";
		if( self:GetXVel() < 0 ) then
			acc = acc * 4;
		end;
		
		self:SetXVel( self:GetXVel() + acc );
	--
	else
	--
		if( self.state == "on_ground" )then
		--
			if( self:GetXVel() < 0 )then
			--
				self:SetXVel( self:GetXVel() + dec );
				if( self:GetXVel() > 0 )then
					self:SetXVel( 0 );
				end;
			--
			elseif( self:GetXVel() > 0 )then
			--
				self:SetXVel( self:GetXVel() - dec );
				if( self:GetXVel() < 0 )then
					self:SetXVel( 0 );
				end;
			--
			end;
		--
		end;
	--
	end;	
--
end;

function CPlayer:UpdateHorz( dt, map )
--
	local xVel = self:GetXVel();
	local xSpd = self:GetXSpeed();
	
	-- jump velocity is calculated in the DoJump function
	xVel = self:GetXVel();
	self:SetXVel( xVel + self:GetXJumpVel() );

	-- collision stuff follows
	xVel = self:GetXVel();
	-- dy is how far we move
	local dx = xVel * dt;
	
	local floorPos = nil;
	
	-- if we're moving right
	if( dx > 0 )then
	--
		self.facing = "right";
		-- start at the left of the player
		local x1 = self:GetXPos();
		-- go all the way over to the right side of the player plus the distance of the move
		local x2 = ( x1 + self:GetWidth() ) + dx;

		-- even though we're considering rightward movement
		-- we have to cover all the tiles the sprite takes up
		-- plus the tiles the movement takes up.  So we also
		-- need the vertical tiles.
		local y1 = self:GetYPos();
		local y2 = ( self:GetYPos() + self:GetHeight() ) - 1;
	
		local wallPos = nil;
		
		floorPos = level:RightsideSlopeCheck( x1, x2, y2 );

		-- adjust y position
		if( floorPos )then
			-- this is basically what causes it to "stick" to the floor
			-- when going up hill while attempting to jump.
			self:SetYPos( floorPos - self:GetHeight() );
		else
			--  try to get a wall position if there's a right side collision
			wallPos = level:GetRightsideCollision( x1, x2, y1, y2 );
			
			-- we've hit a wall
			if( wallPos )then
				self:SetXVel( 0 );
				self:SetXPos( wallPos - self:GetWidth() );
			end;		
		end;

		if( not( wallPos ) )then
			local xPos = self:GetXPos();
			self:SetXPos( xPos + dx );	
		end;
		
	--
	elseif( dx < 0 )then
	--
		self.facing = "left";
		-- start at the left of the player
		local x1 = ( self:GetXPos() + self:GetWidth() ) - 1;
		-- go all the way over to the right side of the player plus the distance of the move
		local x2 = ( x1 - ( self:GetWidth() - 1 ) ) + dx;
		
		-- even though we're considering rightward movement
		-- we have to cover all the tiles the sprite takes up
		-- plus the tiles the movement takes up.  So we also
		-- need the vertical tiles.
		local y1 = self:GetYPos();
		local y2 = ( self:GetYPos() + self:GetHeight() ) - 1;

		--  try to get a wall position if there's a right size collision
		local wallPos = nil;
	
		floorPos = level:LeftsideSlopeCheck( x1, x2, y2 );

		-- adjust y position
		if( floorPos )then
			-- this is basically what causes it to "stick" to the floor
			-- when going up hill while attempting to jump.
			self:SetYPos( floorPos - self:GetHeight() );
		else
			wallPos = level:GetLeftsideCollision( x1, x2, y1, y2 );
			
			-- we've hit a wall
			if( wallPos )then
				self:SetXVel( 0 );
				self:SetXPos( wallPos );
			end;		
		end;
		
		if( not( wallPos ) )then
			local xPos = self:GetXPos();
			self:SetXPos( xPos + dx );	
		end;
	--
	end;

	self.slopeJumpMod = level.slopeJumpMod;	
--
end;

function CPlayer:UpdateVert( dt, map, gravity )
--
	local dGrav = 0;
	dGrav = gravity * dt;
	
	local yVel = self:GetYVel();
	self:SetYVel( yVel + dGrav );

	yVel = self:GetYVel();
	self:SetYVel( yVel + self:GetYJumpVel() );
	
	yVel = self:GetYVel();

	-- dy is how far we move
	local dy = yVel * dt;
	
	-- if we're moving down
	if( dy > 0 )then
	--
		-- we also need the horizontal tiles the sprite covers
		local x1 = self:GetXPos();
		local x2 = ( self:GetXPos() + self:GetWidth() ) - 1;
	
		-- start at the top of the player
		local y1 = self:GetYPos();
		-- go all the way down to the bottom of the player plus the distance of the move
		local y2 = ( y1 + self:GetHeight() ) + dy;
	
		-- grab the current horizontal speed
		local speed = math.abs( self:GetXVel() );
		-- use the speed to determine how far down we look in tiles,
		-- and make sure it's at least 1 so it looks one tile under the sprite
		local extendedLookdown = 
			floor( ( ( speed * TIMESTEP ) / level.map.tileHeight ) + 1 );

		-- we just want to be sure we're on the ground
		-- before we try to snap to the slope, otherwise
		-- just do collisions as per normal.
		if( self.state ~= "on_ground" )then
			extendedLookdown = 0;
		end;
	
		-- check non-sloped tiles first, and if there's no collision check slopes
		local floorPos = level:GetDownwardSlopedCollisions( extendedLookdown, x1, x2, y1, y2 );
		if( not( floorPos ) )then
			floorPos = level:GetDownwardNonSlopedCollisions( x1, x2, y1, y2 );
		end;

		-- we've hit the floor
		if( floorPos )then
			self:SetYVel( 0 );
			self:SetYPos( floorPos - self:GetHeight() );
			self.state = "on_ground";
			self.jumpAngle = level.jumpAngle;
			self.jumpDir:Set( level.jumpDir:GetX(), level.jumpDir:GetY() );
		else
			-- no floor so we can move as far as dy
			local yPos = self:GetYPos();
			self:SetYPos( yPos + dy );
			self.state = "falling";
		end;
		
	--
	elseif( dy < 0 )then
	--
		-- even though we're considering upward movement
		-- we have to cover all the tiles the sprite takes up
		-- plus the tiles the movement takes up.  So we also
		-- need the horizontal tiles.
		local x1 = self:GetXPos();
		local x2 = ( self:GetXPos() + self:GetWidth() ) - 1;
		
		-- start at the top of the player
		local y1 = ( self:GetYPos() + self:GetHeight() ) - 1;
		-- go all the way down to the bottom of the player plus the distance of the move
		local y2 = ( y1 - self:GetHeight() ) + dy; -- dy should be negative which is why we're adding it
		
		--  try to get a ceiling position if there's an upward collision
		local ceilingPos = level:GetUpwardCollisions( x1, x2, y1, y2 );

		-- we've hit the ceiling
		if( ceilingPos )then
			self:SetYVel( 0 );
			self:SetYPos( ceilingPos );
		else
			-- the skys the limit
			local yPos = self:GetYPos();
			self:SetYPos( yPos + dy );
			if( self.isJumping )then
				self.state = "jumping";
			else
				self.state = "flying";
			end;
		end;
	--
	end;
--
end;

function CPlayer:Update( dt, map, gravity )
--
	self:UpdateHorz( dt, map );
	self:UpdateVert( dt, map, gravity );
	self:KeepInBounds();

	--print(self.state);
--
end;

function CPlayer:draw()
--
	local x = self.rect:GetXPos();
	local y = self.rect:GetYPos();
	local w = self:GetWidth(); 
	local h = self:GetHeight();
	love.graphics.rectangle("fill", math.floor(x), math.floor(y), w, h );
--
end;

function CPlayer:Jump(dt, isKeyDown)
--
	-- isKeyDown is null
	if ( isKeyDown ) then
		if ( not ( jump_pressed ) and self.state == "on_ground" ) then
			jump_pressed = true;
			do_jump = true;
		end;
	else
		pressed_time = 0;
		jump_pressed = false;
		do_jump = false;
		self.isJumping = false;
		self:SetXJumpVel( 0 );
		self:SetYJumpVel( 0 );
	end;
	
	if( do_jump ) then
		pressed_time = 
			self:DoJump( dt, pressed_time, MAX_PRESSED_TIME );
	end;	
--
end;

function CPlayer:DoJump( dt, air_time, max_air_time )
--
	if ( air_time < max_air_time ) then
		local jump_remaining = ( max_air_time - air_time );

		local jump_time = 0;

		if ( dt > jump_remaining ) then
			jump_time = jump_remaining;
		else
			jump_time = dt;
		end;
		
		air_time = air_time + dt;

		local jumpForce = self.jumpForce;
		-- jumpDir are the calculated normals that are stored in the tiles, the directional value
		-- is set when there's a collision with said tiles.
		local jumpXVel = ( jumpForce * self.jumpDir:GetX() ) * jump_time;
		local jumpYVel = ( jumpForce * self.jumpDir:GetY() ) * jump_time;
		self:SetXJumpVel( jumpXVel );
		local heightMod = 0;
		local xSpeed = math.abs( self:GetXVel() );
		
		if( self.slopeJumpMod > 0 )then
			heightMod = xSpeed / self.slopeJumpMod;
		end;
		
		self:SetYJumpVel( jumpYVel - heightMod );

		self.isJumping = true;
	else
		self.isJumping = false;
		self:SetXJumpVel( 0 );
		self:SetYJumpVel( 0 );
	end;

	return( air_time );
--
end;

